import sys
import re
import os
import shutil
import subprocess
import argparse
import logging
import requests
import json
import shutil
import tempfile
import glob
import traceback
import requirements
from xml.dom import minidom

GIT_PATH = '/usr/bin/git'
SUPPORTED_TYPES = ['python', 'ruby', 'yarn', 'dotnet', 'nodejs', 'pom']

def find_files(localpath, filename):
    ret_files = []
    for root, subdirs, files in os.walk(localpath):
        for fname in files:
            file_path = os.path.join(root, fname)
            if file_path.endswith(filename):
                ret_files.append(file_path)
    return ret_files

def discover_pom_xml(args, localpath):
    plist = []
    files = find_files(localpath, 'pom.xml')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        contents = fp.read()
        xmldoc = None
        try:
            xmldoc = minidom.parseString(contents)
        except Exception:
            print "Error parsing pom.xml contents"
            return None
        dlist = xmldoc.getElementsByTagName('dependency')
        for d in dlist:
            aid = d.getElementsByTagName('artifactId')[0]
            ver = d.getElementsByTagName('version')[0]
            libname = aid.childNodes[0].data
            libver = ver.childNodes[0].data
            pname = libname + ' ' + libver
            if pname not in plist:
                plist.append(pname)
    return plist 

def discover_package_json(args, localpath):
    plist = []
    files = find_files(localpath, 'package.json')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        contents = fp.read()
        contents = contents.strip()
        cjson = ''
        try:
            cjson = json.loads(contents)
        except Exception:
            print "Error parsing package.json contents"
            return None
        if 'dependencies' in cjson:
            ddict = cjson['dependencies']
            for d in ddict:
                pname = d + ' ' + ddict[d]
                pname = pname.replace('^','')
                pname = pname.replace('~','')
                pname = pname.replace('<','')
                pname = pname.replace('>','')
                pname = pname.replace('=','')
                if pname not in plist:
                    plist.append(pname)
        if 'devDependencies' in cjson:
            ddict = cjson['devDependencies']
            for d in ddict:
                pname = d + ' ' + ddict[d]
                pname = pname.replace('^','')
                pname = pname.replace('~','')
                pname = pname.replace('<','')
                pname = pname.replace('>','')
                pname = pname.replace('=','')
                if pname not in plist:
                    plist.append(pname)
        if 'optionalDependencies' in cjson:
            ddict = cjson['optionalDependencies']
            for d in ddict:
                pname = pname.replace('^','')
                pname = pname.replace('~','')
                pname = pname.replace('<','')
                pname = pname.replace('>','')
                pname = pname.replace('=','')
                pname = d + ' ' + ddict[d]
                if pname not in plist:
                    plist.append(pname)
        return plist 

def discover_packages_config(args, localpath):
    plist = []
    files = find_files(localpath, 'packages.config')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        contents = fp.read()
        xmldoc = None
        try:
            xmldoc = minidom.parseString(contents)
        except Exception:
            print "Error parsing package config contents"
            return None
        plist = xmldoc.getElementsByTagName('package')
        for p in plist:
            libname = p.getAttribute('id')
            libver = p.getAttribute('version')
            pname = libname + ' ' + libver
            if pname not in plist:
                plist.append(pname)
    return plist 

def discover_yarn(args, localpath):
    plist = []
    files = find_files(localpath, 'yarn.lock')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        contents = fp.read()
        cline = contents.splitlines()
        dparse = False
        for index, l in enumerate(cline):
            if l.endswith(':') and 'dependencies' not in l:
                dparse = False
                libname = l.split('@')[0]
                vline = cline[index+1]
                libver = vline.split()[1].replace('"','')
                pname = libname+' '+libver
                if pname not in plist:
                    plist.append(pname)
            if l.endswith(':') and 'dependencies' in l:
                dparse = True
                continue
            if dparse:
                pname = l.strip()
                pname = pname.replace('"','')
                pname = pname.replace('~','')
                if pname == '':
                    dparse = False
                    continue
                if pname not in plist:
                    plist.append(pname)
    return plist 

def discover_ruby(args, localpath):
    plist = []
    files = find_files(localpath, 'gemfile.lock')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        contents = fp.read()
        cline = contents.splitlines()
        specsfound = False
        for index, l in enumerate(cline):
            l = l.strip()
            if l.startswith('specs:'):
                specsfound = True
                continue
            if l == '':
                break
            if specsfound:
                ls = l.split()
                gname = ls[0]
                gver = ''
                if len(ls) > 1:
                    gver = ls[1]
                    gver = re.findall(r'([0-9]+[0-9a-z]*(\.[0-9a-z]+)+)', gver)
                    if gver:
                        gver = gver[0][0]
                    else:
                        gver = ''
                pname = gname + ' ' + gver
                pname = pname.strip()
                if pname not in plist:
                    plist.append(pname)
    return plist 

def discover_python(args, localpath):
    plist = []
    files = find_files(localpath, 'requirements.txt')
    for file_path in files:
        fp = open(file_path, 'r')
        if fp == None:
            continue
        req = requirements.parse(fp)
        for r in req:
            prod = r.name
            if len(r.specs) > 0:
                prod = prod + ' ' + r.specs[0][1]
                if prod not in plist:
                    plist.append(prod)
    return plist

def discover_specified_type(opensource_type, args, localpath):
    if opensource_type not in SUPPORTED_TYPES:
        logging.error("Type not supported")
        sys.exit(1) 

    plist = []
    if opensource_type == 'python':
        plist = discover_python(args, localpath)
    elif opensource_type == 'ruby':
        plist = discover_ruby(args, localpath)
    elif opensource_type == 'yarn':
        plist = discover_yarn(args, localpath)
    elif opensource_type == 'dotnet':
        plist = discover_packages_config(args, localpath)
    elif opensource_type == 'nodejs':
        plist = discover_package_json(args, localpath)
    elif opensource_type == 'pom':
        plist = discover_pom_xml(args, localpath)

    return plist

def get_last_component(repo_path):
    if repo_path.startswith('http:') or repo_path.startswith('https:'):
        return repo_path.rsplit('/',1)[-1]
    else:
        return os.path.basename(os.path.normpath(repo_path))

def discover_inventory(args, localpath):
    default_id_name = get_last_component(args.repo)
    asset_id = None
    if args.assetid == None:
        asset_id = args.type
    else:
        asset_id = args.assetid
    asset_name = None
    if args.assetname == None:
        asset_name = args.type
    else:
        asset_name = args.assetname
    if asset_id is None:
        asset_id = default_id_name
    if asset_name is None:
        asset_name = default_id_name
    asset_id = asset_id.replace('/','-')
    asset_id = asset_id.replace(':','-')
    asset_name = asset_name.replace('/','-')
    asset_name = asset_name.replace(':','-')

    atype = 'Open Source' 
    plist = []
    asset_tags = []
    if args.type is None:
        # If no type is specified, then process all supported types
        for opensource_type in SUPPORTED_TYPES:
            temp_list = discover_specified_type(opensource_type, args, localpath)
            if temp_list is not None and len(temp_list) > 0:
                plist.extend(temp_list)
                asset_tags.append(opensource_type)
    else:
        plist = discover_specified_type(args.type, args, localpath)
        asset_tags.append(args.type)

    if plist == None or len(plist) == 0:
        logging.error("Could not inventory repo "+args.repo)
        sys.exit(1)

    asset_data = {}
    asset_data['id'] = asset_id
    asset_data['name'] = asset_name
    asset_data['type'] = atype
    asset_data['owner'] = args.handle
    asset_data['products'] = plist
    asset_data['tags'] = asset_tags
    
    return [ asset_data ]

def get_inventory(args):
    path = None
    if args.repo.startswith('http'):
        path = tempfile.mkdtemp()
        try:
            cmdarr = [GIT_PATH, 'clone', args.repo, path+'/.']
            out = subprocess.check_output(cmdarr)
        except:
            print traceback.format_exc()
            logging.error('Error cloning repo locally')
            sys.exit(1) 
            os.remove(path)
    elif os.path.isdir(args.repo):
        path = args.repo
    else:
        logging.error('Not a valid repo')
        sys.exit(1) 

    assets = discover_inventory(args, path)

    if args.repo.startswith('http'):
        shutil.rmtree(path)

    return assets



import sys
import platform
import os
import subprocess
import argparse
import logging
import requests
import json
import socket
import csv
import ipaddress

def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP

def get_asset_type(os):
    if "CentOS" in os:
        return "CentOS"
    elif "Red Hat" in os:
        return "Red Hat"
    elif "Ubuntu" in os:
        return "Ubuntu"
    elif "Debian" in os:
        return "Debian"
    elif "Amazon Linux AMI" in os:
        return "Amazon Linux"
    else:
        logging.error('Not a supported OS type')
        return None

def build_ssh_command(args, host, command):
    cmdarr = None
    if host.get('userpwd') is not None and len(host['userpwd']) > 0:
        cmdarr = ["sshpass -p"+host['userpwd']+" ssh "+host['userlogin']+"@"+host['hostname']+" -oStrictHostKeyChecking=no -oCheckHostIP=no \""+command+"\""]
    elif host.get('privatekey') is not None and len(host['privatekey']) > 0:
        cmdarr = ["ssh "+host['userlogin']+"@"+host['hostname']+" -i "+host['privatekey']+" -oStrictHostKeyChecking=no -oCheckHostIP=no \""+command+"\""]
    else:
        cmdarr = ["ssh "+host['userlogin']+"@"+host['hostname']+" -oStrictHostKeyChecking=no -oCheckHostIP=no \""+command+"\""]
    return cmdarr

def get_os_release(args, host):
    cmdarr = ["/bin/cat /etc/os-release"]
    if host['remote']:
        cmdarr = build_ssh_command(args, host, cmdarr[0])
        if cmdarr is None:
            logging.error("Error determining OS release")
            return None

    out = ''
    try:
        out = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error determining OS release")
        return None 
    for l in out.splitlines():
        if 'PRETTY_NAME' in l:
            return l.split('=')[1].replace('"','')
    return None

def discover_rh(args, host):
    plist = []
    cmdarr = ["/usr/bin/yum list installed"]
    if host['remote']:
        cmdarr = build_ssh_command(args, host, cmdarr[0])
        if cmdarr is None:
            logging.error("Error running inventory")
            return None

    logging.info("Retrieving product details")
    yumout = ''
    try:
        yumout = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error running inventory")
        return None 

    begin = False
    for l in yumout.splitlines():
        if 'Installed Packages' in l:
            begin = True
            continue
        if not begin:
            continue
        lsplit = l.split()
        pkg = lsplit[0]
        if len(lsplit) > 1:
            ver = lsplit[1]
        else:
            ver = ''
        pkgsp = pkg.split(".")
        pkg = pkgsp[0]
        arch = pkgsp[1]
        if ':' in ver:
            ver = ver.split(':')[1]
        ver = ver + "." + arch
        logging.debug("Found product [%s %s]", pkg, ver)
        plist.append(pkg+' '+ver)
    logging.info("Completed retrieval of product details")
    return plist

def discover_ubuntu(args, host):
    plist = []
    cmdarr = ["/usr/bin/apt list --installed"]
    if host['remote']:
        cmdarr = build_ssh_command(args, host, cmdarr[0])
        if cmdarr is None:
            logging.error("Error running inventory")
            return None

    logging.info("Retrieving product details")
    yumout = ''
    try:
        yumout = subprocess.check_output(cmdarr, shell=True)
    except subprocess.CalledProcessError:
        logging.error("Error running inventory")
        return None 

    begin = False
    for l in yumout.splitlines():
        if 'Listing...' in l:
            begin = True
            continue
        if not begin:
            continue
        if l.strip() == '':
            continue
        lsplit = l.split()
        pkg = lsplit[0].split('/')[0]
        ver = lsplit[1]
        logging.debug("Found product [%s %s]", pkg, ver)
        plist.append(pkg+' '+ver)
    logging.info("Completed retrieval of product details")
    return plist

def discover(args):
    handle = args.handle
    token = args.token
    instance = args.instance

    if args.remote_hosts_csv is not None:
        with open(args.remote_hosts_csv, mode='r') as csv_file:
            csv_reader = csv.DictReader(csv_file, quoting=csv.QUOTE_NONE, escapechar='\\')
            remote_hosts = []
            for row in csv_reader:
                if '/' in row['hostname']: # CIDR is specified, then expand it
                    logging.info("Enumerating IPs based on specified CIDR [%s]", row['hostname'])
                    net = ipaddress.ip_network(unicode(row['hostname'],"ascii"))
                    for a in net:
                        trow = row.copy()
                        trow['hostname'] = str(a)
                        remote_hosts.append(trow)
                        remote_hosts[-1]['remote'] = True
                        logging.info("Enumerated IP: %s", a)
                else:
                    remote_hosts.append(row)
                    remote_hosts[-1]['remote'] = True
        return discover_hosts(args, remote_hosts)
    else:
        host = { }
        host['assetid'] = get_ip() if args.assetid is None else args.assetid
        host['assetname'] = host['assetid'] if args.assetname is None else args.assetname
        host['remote'] = False
        hosts = [ host ]
        return discover_hosts(args, hosts)

def discover_hosts(args, hosts):

    assets = []
    for host in hosts:
        asset = discover_host(args, host)
        if asset is not None:
            assets.append(asset)
    return assets

def discover_host(args, host):

    asset_id = host['assetid'] if host.get('assetid') is not None else host['hostname']
    asset_name = host['assetname'] if host.get('assetname') is not None else asset_id

    asset_id = asset_id.replace('/','-')
    asset_id = asset_id.replace(':','-')
    asset_name = asset_name.replace('/','-')
    asset_name = asset_name.replace(':','-')

    logging.info("Started inventory discovery for asset [%s]", asset_id)

    os = get_os_release(args, host)
    if os is None:
        logging.error("Failed to identify OS for asset [%s]", asset_id)
        return None

    atype = get_asset_type(os)
    if atype is None:
        logging.error("Could not determine asset type for asset [%s]", asset_id)
        return None

    plist = None
    if atype == 'CentOS' or atype == 'Red Hat' or atype == 'Amazon Linux':
        plist = discover_rh(args, host)
    elif atype == 'Ubuntu' or atype == 'Debian':
        plist = discover_ubuntu(args, host)

    if plist == None or len(plist) == 0:
        logging.error("Could not inventory asset [%s]", asset_id)
        return None

    asset_data = {}
    asset_data['id'] = asset_id
    asset_data['name'] = asset_name
    asset_data['type'] = atype
    asset_data['owner'] = args.handle
    asset_data['products'] = plist
    asset_tags = []
    asset_tags.append('OS_RELEASE:' + os)
    asset_tags.append('Linux')
    asset_tags.append(atype)
    asset_data['tags'] = asset_tags

    logging.info("Completed inventory discovery for asset [%s]", asset_id)

    return asset_data

def get_inventory(args):
    return discover(args)



import sys
import json
import boto3
import difflib
import codecs
import os
import subprocess
import re
import logging
import argparse
import requests

# Prints details about subscriptions, resource groups and workspaces
def print_details(token):
    allsubs = get_all_subscriptions(token)
    print ""
    print "Missing details for subscription/resource group/workspace...."
    print "Available subscriptions with resource group and workspace details as below:"
    for sub in allsubs:
        print "Subscription: %s" % sub
        resourcegroups = get_all_resourcegroups_for_subscription(sub, token)
        for res_group in resourcegroups:
            print " ** Resource group: %s" % res_group
        workspaces = get_all_workspaces_for_subscription(sub, token)
        for workspace in workspaces:
            print " ** Workspace: %s" % workspace
    print ""
    print "Please re-run twigs with appropriate values for subscription, resource group and workspace."
    print ""

# Main entry point
def get_inventory(args):
    params =  {}
    params['handle'] = args.handle
    params['tenant_id'] = args.azure_tenant_id
    params['app_id'] = args.azure_application_id
    params['app_key'] = args.azure_application_key
    params['subscription'] = args.azure_subscription
    params['resource_group'] = args.azure_resource_group
    params['workspace'] = args.azure_workspace
    params['enable_tracking_tags'] = args.enable_tracking_tags
    token = get_access_token(params)
    if token is None:
        return
    params['access_token'] = token
    if args.azure_subscription is None or args.azure_resource_group is None or args.azure_workspace is None:
        print_details(token)
        return
    assets = retrieve_inventory(params)
    return assets

def parse_inventory(email,data,params):
    logging.info("Processing inventory retrieved from Azure...")
    hosts = []
    assets = []
    asset_map = {}
    for i in range(len(data)):
        if data[i][4] == 'WindowsServices': #ConfigDataType
            continue
        host = data[i][5]
        publisher = data[i][2]

        if host not in hosts  and publisher != '0':
            patches = []
            products = []
            asset_map = {}
            asset_map['owner'] = email
            asset_map['host'] = host
            asset_map['id'] = host
            asset_map['name'] = host
            asset_map['tags'] = [ ]
            asset_map['patch_tracker'] = { } # To help remove duplicate patches
            if data[i][1] == 'Update': #ApplicationType for MS patches
                patch = parse_patch(data[i])
                patches.append(patch)
                asset_map['patch_tracker'][patch['id']] = patch['id']
            if data[i][1] == 'Package' or data[i][1] == 'Application': #ApplicationType for Linux packages
                pname = data[i][0]
                pversion =  data[i][3]
                products.append(pname+' ' + pversion)
            asset_map['products'] = products
            asset_map['patches'] = patches
            os = get_os_name(host,params)
            asset_map['type'] = get_os_type(os)
            if len(asset_map['type']) > 0:
                asset_map['tags'].append(asset_map['type'])
            if asset_map['type'] == 'Windows':
                asset_map['tags'].append('OS_RELEASE:' + os)
            if params['enable_tracking_tags'] == True:
                asset_map['tags'].append("SOURCE:Azure:" + params['tenant_id'])
            else:
                asset_map['tags'].append("SOURCE:Azure")
            assets.append(asset_map)
            hosts.append(host)
        else:
            for asset in assets:
                if asset['host'] == host:
                    products = asset['products']
                    patches = asset['patches']
                    if data[i][1] == 'Update': #ApplicationType for MS patches
                        patch = parse_patch(data[i])
                        if asset['patch_tracker'].get(patch['id']) is None:
                            patches.append(patch)
                            asset['patches'] = patches
                            asset['patch_tracker'][patch['id']] = patch['id']
                    if data[i][1] == 'Package' or data[i][1] == 'Application': #ApplicationType for Linux packages
                        pname = data[i][0]
                        pversion =  data[i][3]
                        products.append(pname+' ' + pversion)
                        asset['products'] = products
    # Remove the additional field 'patch_tracker' (added to avoid duplicate patches)
    for asset in assets:
        asset.pop('patch_tracker', None)
    return assets

def parse_patch(data):
    patch_id = re.findall(r'(KB[0-9]+)', data[0])
    patch = {}
    patch['url'] = ''
    patch['id'] = patch_id[0]
    patch['product'] = ''
    patch['description'] = data[0]
    return patch
            
def get_os_type(ostype):
    if ostype is None:
        return ''
    if 'Microsoft' in  ostype or 'Windows' in ostype:
        return 'Windows'
    if 'Red Hat' in ostype or 'redhat' in ostype:
        return 'Red Hat'
    return ''

def retrieve_inventory(params):
    email = params['handle']
    sub_id = params['subscription']
    resource_group = params['resource_group']
    workspace_id = params['workspace']
    token = params['access_token']
    CURL = '/usr/bin/curl --silent -H "Content-Type:application/json" -H "Authorization:Bearer %s" -d ' % token
    URL = '-X POST https://management.azure.com/subscriptions/%s/resourceGroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s/api/query?api-version=2017-01-01-preview' %(sub_id,resource_group,workspace_id)
    DATA = '{"query":"ConfigurationData | summarize by SoftwareName, SoftwareType, Publisher, CurrentVersion, ConfigDataType, Computer"}' 
    cmd = CURL + "'" + DATA + "'" + " " + URL

    logging.info("Retrieving inventory details from Azure...") 

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        print "Could not fetch threat updates"
        sys.exit(1)

    if output == "Kill":
        print "Invalid tw instance...terminating"
        sys.exit(1)

    response = json.loads(output)
    if response.get('Tables'):
        tables = response['Tables']
        return parse_inventory(email,tables[0]['Rows'],params)

#Get access token using  an AAD, an app id associted with that AAD and the API key/secret for that app
def get_access_token(params):         
    aad_id = params['tenant_id']
    aad_app_id = params['app_id']
    app_key = params['app_key']
    CURL = '/usr/bin/curl --silent -d '
    URL = "-X POST https://login.microsoftonline.com/" + aad_id + "/oauth2/token"
    DATA = 'grant_type=client_credentials&client_id=%s&client_secret=%s&resource=https://management.azure.com/' % (aad_app_id,app_key)

    cmd = CURL + "'" + DATA + "'"  + " " + URL

    logging.info("Getting access token...") 

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        print "Could not fetch threat updates"
        return '' 

    if output == "Kill":
        return '' 
    try:
        response = json.loads(output)
        token = response['access_token']
    except KeyError:
        error = response['error_description']
        print error
        return None
    return token

# Try to get OS details for given VM
def get_os_name(host,params):
    CURL = '/usr/bin/curl --silent -H "Content-Type:application/json" -H "Authorization:Bearer %s" ' % params['access_token']
    URL = "-X GET \"https://management.azure.com/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/virtualMachines/%s/instanceView?api-version=2018-06-01\"" % (params['subscription'], params['resource_group'],host)
    cmd = CURL + URL

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        return None

    if output == "Kill":
        return None

    response = json.loads(output)
    return response.get('osName')

# Get a list of subscriptions for current AAD/Tenant
def get_all_subscriptions(token):
    allsubs = []
    CURL = '/usr/bin/curl --silent -H "Content-Type:application/json" -H "Authorization:Bearer %s" ' % token
    URL = "-X GET https://management.azure.com/subscriptions?api-version=2018-01-01"
    cmd = CURL + URL

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        print "Could not fetch threat updates"
        sys.exit(1)

    if output == "Kill":
        print "Invalid tw instance...terminating"
        sys.exit(1)

    subs = json.loads(output)
    sublist = subs['value']
    for sub in sublist:
        allsubs.append(sub['subscriptionId'])
    return allsubs

#Get all resource groups for a subscription
def get_all_resourcegroups_for_subscription(subid,token):
    allresourcegroups = []
    CURL = '/usr/bin/curl --silent -H "Content-Type:application/json" -H "Authorization:Bearer %s" ' % token
    URL = '-X GET https://management.azure.com/subscriptions/%s/resourcegroups?api-version=2018-01-01' % subid
    cmd = CURL + URL

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        print "Could not fetch resourcegroups"
        return []
       
    if output == "Kill":
        print "Could not fetch resourcegorups"
        return []

    rgroups = json.loads(output)
    rlist = rgroups['value']
    for r in rlist:
        allresourcegroups.append(r['name'])
    return allresourcegroups

#Get all workspaces for the subscription
def get_all_workspaces_for_subscription(subid,token):
    allworkspaces = []
    CURL = '/usr/bin/curl --silent -H "Content-Type:application/json" -H "Authorization:Bearer %s" ' % token
    URL = '-X GET https://management.azure.com/subscriptions/%s/providers/Microsoft.OperationalInsights/workspaces?api-version=2017-01-01-preview' % subid
    cmd = CURL + URL

    p = subprocess.Popen(cmd, bufsize=8192, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    pstat = p.wait()

    if output == "Error":
        print "Could not get workspaces"
        return []

    if output == "Kill":
        print "Could not get workspaces"
        return []

    workspaces = json.loads(output)
    wlist = workspaces['value']
    for w in wlist:
        allworkspaces.append(w['name'])
    return allworkspaces

